@@ -6,6 +6,7 @@
 import pickle  # nosec
 import re
 import signal
+import warnings
 from multiprocessing import Pool, cpu_count
 from multiprocessing.pool import AsyncResult
 from pathlib import Path
@@ -180,25 +181,21 @@ class QumaResult:
         """pd.DataFrame: dataframe of quma methylation analysis values."""
 
     @staticmethod
-    def _process_raw_quma(quma_result: str) -> List[str]:
+    def _process_raw_quma(quma_data: List[quma.Reference]) -> List[str]:
         """Spit and process raw quma results into a pandas dataframe.
 
         Args:
-            quma_result (str): console output from quma program.
+            quma_data (List[quma.Reference]): list of quma References.
 
         Returns:
             pd.DataFrame: intermediate dataframe to append
         """
 
-        dots: List[str] = []
-        for line in quma_result.splitlines():
-            if not line.lstrip().startswith("g"):
-                fields: List[str] = line.split("\t")
-                if float(fields[7]) < MIN_PERCENT_ALIGNMENT:
-                    dots.append("FAIL")
-                else:
-                    dots.append(fields[13])
-        # Sort dots output by number of "1"
+        dots = [
+            res.res.val if res.res.perc > MIN_PERCENT_ALIGNMENT else "FAIL"
+            for res in quma_data
+        ]
+
         return sorted(dots, key=lambda t: t.count("1"))
 
     def _pool_processing(self) -> pd.DataFrame:
@@ -259,7 +256,7 @@ class QumaResult:
 
         quma_obj: quma.Quma = self._access_quma(genomic_contents, read_contents)
 
-        processed_quma: List[str] = self._process_raw_quma(quma_obj.values)
+        processed_quma: List[str] = self._process_raw_quma(quma_obj.data)
         # Next, add this readname to the holding data frame
         int_df: pd.DataFrame = pd.DataFrame({position: processed_quma})
         return int_df, quma_obj
@@ -317,17 +314,24 @@ class GenomicPositionData:
         self.quma_results: Dict[str, QumaResult] = self._quma_full_threaded()
         """Dict[str, QumaResult]: list of QumaResults."""
 
-        self.means: pd.DataFrame = self._process_means()
-        """pd.DataFrame: dataframe of mean methylation values."""
+        with tqdm(total=4, desc="Summary Statistics") as pbar:
+            self.means: pd.DataFrame = self._alt_process_means()
+            """pd.DataFrame: dataframe of mean methylation values."""
+            pbar.update(1)
 
-        self.modes: pd.DataFrame = self._process_modes()
-        """pd.DataFrame: dataframe of modes of methylation values."""
+            self.modes: pd.DataFrame = self._alt_process_modes()
+            """pd.DataFrame: dataframe of modes of methylation values."""
+            pbar.update(1)
 
-        self.diffs: pd.DataFrame = self._find_diffs(self.means, self.modes)
-        """pd.DataFrame: dataframe of difference mean minus mode methylation values."""
+            self.diffs: pd.DataFrame = self._find_diffs(self.means, self.modes)
+            """pd.DataFrame: df of difference mean minus mode methylation values."""
+            pbar.update(1)
 
-        self.individual_data: pd.DataFrame = self._return_individual_data()
-        """pd.DataFrame: dataframe of individual methylation values."""
+            self.individual_data: pd.DataFrame = self._alt_return_individual_data()
+            """pd.DataFrame: dataframe of individual methylation values."""
+
+            self.positions = self.means.columns.tolist()
+            pbar.update(1)
 
     def _index_and_fetch(self) -> Dict[str, BamOutput]:
         """Wrapper to call processing of each sam file.
@@ -457,6 +461,33 @@ class GenomicPositionData:
 
         return means_df
 
+    def _alt_process_means(self) -> pd.DataFrame:
+        """Process the mean values at each individual position for each cell line.
+
+        Returns:
+            pd.DataFrame: dataframes of mean values for each position in each cell line.
+        """
+
+        cells: List[str] = []
+        for each in self.cell_types:
+            match: Optional[Match[str]] = re.match(
+                self.config.fname_pattern, Path(each).name
+            )
+            cell_line_name: str
+            if isinstance(match, Match):
+                cell_line_name = match.group(1)
+            else:
+                cell_line_name = Path(each).name
+            cells.append(cell_line_name)
+        df_list = {
+            cell: self.return_individual_positions(cell).apply(pd.to_numeric).mean()
+            for cell in cells
+        }
+
+        df = pd.DataFrame.from_dict(df_list).T
+        df.columns = df.columns.astype(str)
+        return df
+
     def _process_modes(self) -> pd.DataFrame:
         """Process the mode values at each position for each cell line.
 
@@ -483,6 +514,37 @@ class GenomicPositionData:
 
         return modes_df
 
+    def _alt_process_modes(self) -> pd.DataFrame:
+        """Process the mode at each individual position for each cell line.
+
+        Returns:
+            pd.DataFrame: dataframes of mode values for each individual position.
+        """
+
+        cells: List[str] = []
+        for each in self.cell_types:
+            match: Optional[Match[str]] = re.match(
+                self.config.fname_pattern, Path(each).name
+            )
+            cell_line_name: str
+            if isinstance(match, Match):
+                cell_line_name = match.group(1)
+            else:
+                cell_line_name = Path(each).name
+            cells.append(cell_line_name)
+
+        df_list = {
+            cell: self.return_individual_positions(cell)
+            .apply(pd.to_numeric)
+            .mode()
+            .iloc[0]
+            for cell in cells
+        }
+
+        df = pd.DataFrame.from_dict(df_list).T
+        df.columns = df.columns.astype(str)
+        return df
+
     def _return_individual_data(self) -> pd.DataFrame:
         """Return a dataframe for methylation values at each position for each cell line.
 
@@ -505,6 +567,45 @@ class GenomicPositionData:
 
         return working_df
 
+    def _alt_return_individual_data(self) -> pd.DataFrame:
+        """Return a dataframe for methylation values at each position for each cell line.
+
+        Returns:
+            pd.DataFrame: methylation values for each position in each cell line
+        """
+
+        cells: List[str] = []
+        for each in self.cell_types:
+            match: Optional[Match[str]] = re.match(
+                self.config.fname_pattern, Path(each).name
+            )
+            cell_line_name: str
+            if isinstance(match, Match):
+                cell_line_name = match.group(1)
+            else:
+                cell_line_name = Path(each).name
+            cells.append(cell_line_name)
+        df_list: List[pd.DataFrame] = []
+        for cell in cells:
+            ind_pos: pd.DataFrame = self.return_individual_positions(cell).apply(
+                pd.to_numeric
+            )
+            ind_pos = ind_pos.stack().reset_index(level=0, drop=True)
+            ind_pos = (
+                ind_pos.groupby(ind_pos.index)
+                .apply(list)
+                .to_frame()
+                .transpose()
+                .rename(index={0: cell})
+            )
+
+            df_list.append(ind_pos)
+
+        # print(repr(df_dict))
+        df = pd.concat(df_list)
+        df.columns = df.columns.astype(str)
+        return df
+
     def _return_read_values(
         self,
         pos: str,
@@ -623,11 +724,15 @@ class GenomicPositionData:
 
         Raises:
             ValueError: invalid plotting backend
+            ValueError: No data available at that position
         """
         data = self.individual_data
         if not backend:
             backend = self.config.viz_backend
 
+        if np.all(np.isnan(data.loc[cell_line, position])):
+            raise ValueError("No data available at that position")
+
         if backend == "plotly":
             fig: go.Figure = viz._create_histogram(data, cell_line, position, backend)
             fig.show()
@@ -804,7 +909,9 @@ class GenomicPositionData:
             for column in row.index:
                 value = row[column]
                 if np.all(pd.notnull(value)):
-                    good, stat, crits = self._anderson_darling_test(value)
+                    with warnings.catch_warnings():
+                        warnings.simplefilter("ignore", RuntimeWarning)
+                        good, stat, crits = self._anderson_darling_test(value)
                     if good:
                         sig_dict["diff"].append(self.diffs.loc[index, column])
                         sig_dict["cellLine"].append(index)
@@ -835,6 +942,118 @@ class GenomicPositionData:
             return AD_stats(is_sig, stat, crits)
         return AD_stats(False, np.nan, [np.nan])
 
+    def return_individual_positions(self, cell_line: str) -> pd.DataFrame:
+        """Return a dataframe of methylation at genomic position resolution.
+
+        Args:
+            cell_line (str): Cell line to analyze.
+
+        Returns:
+            pd.DataFrame: dataframe of methylation at genomic position resolution.
+        """
+
+        class PointData(NamedTuple):
+            """Named tuple to cointain individual Quma point data."""
+
+            cell: str
+            read_position: int
+            read: str
+            positions: List[int]
+            values: List[str]
+
+        def _find_positions(
+            cell: str, pos: str, data_list: List[quma.Reference], gseq: str
+        ) -> List[PointData]:
+            """Find individual resolution positions and data in quma results.
+
+            Args:
+                cell (str): cell line.
+                pos (str): position of read.
+                data_list (List[quma.Reference]): list of quma read data
+                gseq (str): reference genomic sequence of read position
+
+            Returns:
+                List[PointData]: list of data points from quma results.
+            """
+            output: List[PointData] = []
+            for each in data_list:
+                read: str = each.fasta.com
+                offset: List[int] = [
+                    m.start() for m in re.finditer(each.res.gAli, gseq)
+                ]
+                real_off_set: int
+                if offset:
+                    real_off_set = offset[0]
+                else:
+                    real_off_set = 0
+                init_pos_list: List[int] = [
+                    m.start() for m in re.finditer("CG", each.res.gAli)
+                ]
+                pos_list: List[int] = [
+                    (this + int(pos) + real_off_set) for this in init_pos_list
+                ]
+
+                my_point: PointData = PointData(
+                    cell=cell,
+                    read_position=int(pos),
+                    read=read,
+                    positions=pos_list,
+                    values=list(each.res.val),
+                )
+                output.append(my_point)
+
+            return output
+
+        def _collate_positions(output: List[PointData]) -> Dict[int, List[str]]:
+            """Collect together list of quma data points into position-indexed dict.
+
+            Args:
+                output (List[PointData]): list of quma read data points
+
+            Returns:
+                Dict[int, str]: position-indexed genomic resolution methylation dict.
+            """
+            result_dict: Dict[int, List[str]] = {}
+            for each in output:
+                for idx, val in enumerate(each.positions):
+                    try:
+                        my_val: str = each.values[idx]
+                    except IndexError:
+                        pass
+                    if my_val in ["1", "0"]:
+                        if result_dict.get(val):
+                            result_dict[val].append(my_val)
+                        else:
+                            result_dict[val] = [my_val]
+            return result_dict
+
+        def _make_pos_df(data: Dict[int, List[str]]) -> pd.DataFrame:
+            """Convert dictionary of genomic resolution data into dataframe.
+
+            Args:
+                data (Dict[int, List[str]]): pos-indexed high res. methylation dict.
+
+            Returns:
+                pd.DataFrame: dataframe of pos-indexed high resolution methylation.
+            """
+            int_df: pd.DataFrame = pd.DataFrame()
+            for k, v in data.items():
+                int_df = pd.concat([int_df, pd.DataFrame({k: sorted(v)})], axis=1)
+            return int_df
+
+        query: QumaResult = self.quma_results[cell_line]
+        positions: List[str] = query._positions
+        data: List[quma.Quma] = query.quma_output
+        big_list: List[List[PointData]] = []
+        for pos, dat in zip(positions, data):
+            big_list.append(_find_positions("NCIH2171", pos, dat.data, dat._gseq))
+
+        final_list: List[PointData] = [item for sublist in big_list for item in sublist]
+        full_data: Dict[int, List[str]] = _collate_positions(final_list)
+
+        final_df: pd.DataFrame = _make_pos_df(full_data)
+        return final_df
+
 
 ##################################################################################
 ##################################################################################
