@@ -16,9 +16,12 @@ import pytest
 from pyllelic import pyllelic
 
 from .inputs import (
+    EXPECTED_ALLELIC_DATA,
+    EXPECTED_BAM_INDIVIDUAL_POSITIONS,
     EXPECTED_BAM_OUTPUT_GENOME_VALUES,
     EXPECTED_BAM_OUTPUT_POSITIONS,
     EXPECTED_BAM_OUTPUT_VALUES,
+    EXPECTED_INDIVIDUAL_POSITIONS,
     EXPECTED_INTERMEDIATE_DIFFS,
     EXPECTED_INTERMEDIATE_INDIVIDUAL_DATA,
     EXPECTED_MEANS,
@@ -157,13 +160,8 @@ def test_pyllelic(tmp_path_factory):
     config = setup_config(p)
     INPUT_BAM_LIST = ["fh_test_tissue.bam"]
     genomic_position_data = pyllelic.pyllelic(config=config, files_set=INPUT_BAM_LIST)
-    positions = []
-    for each in EXPECTED_BAM_OUTPUT_POSITIONS:
-        positions.append(each)
 
-    EXPECTED_POSITIONS = sorted(set(positions))
-
-    assert genomic_position_data.positions == EXPECTED_POSITIONS
+    assert genomic_position_data.positions == EXPECTED_BAM_INDIVIDUAL_POSITIONS
     assert genomic_position_data.cell_types == [str(p / "test" / "fh_test_tissue.bam")]
 
 
@@ -261,15 +259,20 @@ class Test_QumaOutput:
     def test_process_raw_quma(self, set_up_quma_output):
         quma_output = set_up_quma_output
 
-        EXPECTED = ["1A", "11"]
-        actual = quma_output._process_raw_quma(TEST_QUMA_RESULT)
+        quma_reference = quma_output.quma_output[0].data
+
+        EXPECTED = ["111", "111", "111", "111", "111", "1111"]
+        actual = quma_output._process_raw_quma(quma_reference)
         assert EXPECTED == actual
 
     def test_process_raw_quma_below_min_alignment(self, set_up_quma_output):
         quma_output = set_up_quma_output
-        TEST_QUMA_RESULT_FAIL = TEST_QUMA_RESULT[:157] + "4" + TEST_QUMA_RESULT[158:]
-        EXPECTED = ["FAIL", "11"]
-        actual = quma_output._process_raw_quma(TEST_QUMA_RESULT_FAIL)
+
+        quma_reference = quma_output.quma_output[0].data
+        quma_reference[0].res.perc = 10
+
+        EXPECTED = ["FAIL", "111", "111", "111", "111", "1111"]
+        actual = quma_output._process_raw_quma(quma_reference)
         assert EXPECTED == actual
 
     def test__pool_processing(self, set_up_quma_output):
@@ -327,13 +330,8 @@ class Test_GenomicPositionData:
 
     def test_init(self, set_up_genomic_position_data):
         p, genomic_position_data = set_up_genomic_position_data
-        positions = []
-        for each in EXPECTED_BAM_OUTPUT_POSITIONS:
-            positions.append(each)
 
-        EXPECTED_POSITIONS = sorted(set(positions))
-
-        assert genomic_position_data.positions == EXPECTED_POSITIONS
+        assert genomic_position_data.positions == EXPECTED_BAM_INDIVIDUAL_POSITIONS
         assert genomic_position_data.cell_types == [
             str(p / "test" / "fh_test_tissue.bam")
         ]
@@ -486,60 +484,6 @@ class Test_GenomicPositionData:
 
     def test_summarize_allelelic_data(self, set_up_genomic_position_data):
         _, genomic_position_data = set_up_genomic_position_data
-        EXPECTED = pd.DataFrame(
-            {
-                "cellLine": {0: "test"},
-                "position": {0: "1295321"},
-                "ad_stat": {0: 13.510575364459704},
-                "p_crit": {0: 1.009},
-                "diff": {0: -0.04878048780487809},
-                "raw": {
-                    0: [
-                        0.0,
-                        0.5,
-                        0.5,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                        1.0,
-                    ]
-                },
-            }
-        )
 
         with np.testing.suppress_warnings() as sup:  # ignore degrees of freedom warning
             sup.filter(RuntimeWarning, "Degrees of freedom")
@@ -550,8 +494,8 @@ class Test_GenomicPositionData:
             )
             actual2 = genomic_position_data.summarize_allelic_data()
 
-        pd.testing.assert_frame_equal(EXPECTED, actual1)
-        pd.testing.assert_frame_equal(EXPECTED, actual2)
+        pd.testing.assert_frame_equal(EXPECTED_ALLELIC_DATA, actual1)
+        pd.testing.assert_frame_equal(EXPECTED_ALLELIC_DATA, actual2)
 
     def test_sig_methylation_differences(self, set_up_genomic_position_data, mocker):
         _, genomic_position_data = set_up_genomic_position_data
@@ -685,3 +629,10 @@ class Test_GenomicPositionData:
         EXPECTED_TRUNCATED_DIFF = TEST_DIFFS.dropna(how="all")
         actual = pyllelic.GenomicPositionData._truncate_diffs(TEST_DIFFS)
         pd.testing.assert_frame_equal(actual, EXPECTED_TRUNCATED_DIFF)
+
+    def test_return_individual_positions(self, set_up_genomic_position_data):
+        _, genomic_position_data = set_up_genomic_position_data
+
+        actual = genomic_position_data.return_individual_positions("test")
+
+        pd.testing.assert_frame_equal(actual, EXPECTED_INDIVIDUAL_POSITIONS)
